@@ -15,10 +15,10 @@ __all__ = ['BaseService', 'BaseServiceTask']
 class BaseService(BaseDaemon):
 
     # interval in sec to check for tasks
-    check_interval = 5
+    task_check_interval = 5
 
     # delay in sec to wait for tasks threads exit
-    task_exit_delay = 3
+    task_exit_delay = 5
 
     def __init__(self, name=None, logger=None, debug=0):
         super(BaseService, self).__init__(
@@ -44,49 +44,28 @@ class BaseService(BaseDaemon):
                 raise RuntimeError("Invalid task: %s" % str(T))
         # debug tasks
         if self.debug >= 2:
-            self.log.debug("Loaded tasks: (%s)"
-                           % ','.join([T.__name__ for T in self.tasks]))
+            self.log.debug(
+                "Loaded tasks: (%s)"
+                % ','.join([T.__name__ for T in self.tasks]))
 
     def execute(self):
-        for T in self.tasks:
-            try:
-                # check and clean dead tasks threads
-                t = self._threads.get(T.__name__, None)
-                if t and not t.is_alive():
-                    del(self._threads[T.__name__])
-                    self.log.warning("found dead <TASK:%s>" % T.__name__)
-
-                # stop suspended task
-                if T.__name__ in self._threads and \
-                        T.__name__ in self._suspended:
-                    self.stop_task(T.__name__)
-
-                # start new task
-                if T.__name__ not in self._threads and \
-                        T.__name__ not in self._suspended:
-                    self.start_task(T.__name__)
-
-            except Exception:
-                self.log.error(format_exc().strip())
+        self.check_tasks()
 
         # checking threads interval
-        self.sleep(self.check_interval)
+        self.sleep(self.task_check_interval)
 
     def terminate(self):
-        try:
-            self.log.info("stopping all tasks")
-            for name in list(self._threads.keys()):
-                self._threads[name].stop()
+        self.log.debug("wait all tasks exit")
+        for name in list(self._threads.keys()):
+            if self._threads[name].is_alive():
+                self._threads[name].join(self.task_exit_delay)
 
-            self.log.debug("wait all tasks exit")
-            for name in list(self._threads.keys()):
-                if self._threads[name].is_alive():
-                    self._threads[name].join(self.task_exit_delay)
+    def stop(self):
+        self.term_event.set()
 
-        except Exception:
-            self.log.error(format_exc().strip())
-
-        self.log.info("exit")
+        self.log.info("stopping all tasks")
+        for name in list(self._threads.keys()):
+            self._threads[name].stop()
 
     def get_task(self, name):
         for T in self.tasks:
@@ -133,6 +112,28 @@ class BaseService(BaseDaemon):
             return False
         return self.start_task(name)
 
+    def check_tasks(self):
+        for T in self.tasks:
+            try:
+                # check and clean dead tasks threads
+                t = self._threads.get(T.__name__, None)
+                if t and not t.is_alive():
+                    del(self._threads[T.__name__])
+                    self.log.warning("found dead <TASK:%s>" % T.__name__)
+
+                # stop suspended task
+                if T.__name__ in self._threads and \
+                        T.__name__ in self._suspended:
+                    self.stop_task(T.__name__)
+
+                # start new task
+                if T.__name__ not in self._threads and \
+                        T.__name__ not in self._suspended:
+                    self.start_task(T.__name__)
+
+            except Exception:
+                self.log.error(format_exc().strip())
+
 
 class BaseServiceTask(threading.Thread):
 
@@ -162,17 +163,26 @@ class BaseServiceTask(threading.Thread):
         pass
 
     def run(self):
-        # initialize task
-        self.initialize()
+        try:
+            # initialize task
+            self.initialize()
+            # run task forever
+            while not self.term_event.is_set():
+                self.execute()
+        except Exception:
+            self.log.error(format_exc().strip())
+        except (KeyboardInterrupt, SystemExit):
+            pass
 
-        # run task forever
-        while not self.term_event.is_set():
-            self.execute()
+        try:
+            # terminate task
+            self.terminate()
+        except Exception:
+            self.log.error(format_exc().strip())
+        except (KeyboardInterrupt, SystemExit):
+            pass
 
-        self.term_event.clear()
-
-        # terminate task
-        self.terminate()
+        self.log.info("terminated")
 
     def start(self):
         super(BaseServiceTask, self).start()
@@ -181,4 +191,5 @@ class BaseServiceTask(threading.Thread):
         self.term_event.set()
 
     def sleep(self, timeout):
-        self.term_event.wait(timeout=timeout)
+        if self.term_event.wait(timeout=timeout):
+            raise SystemExit()
